@@ -505,6 +505,22 @@ get_ical_uid (icalcomponent *ical)
   return g_strdup (icalcomponent_get_uid (ical));
 }
 
+static char *
+get_ical_rid (icalcomponent *ical)
+{
+  icalproperty        *prop;
+  struct icaltimetype  ical_time;
+  
+  prop = icalcomponent_get_first_property (ical, ICAL_RECURRENCEID_PROPERTY);
+  if (!prop)
+    return NULL;
+
+  ical_time = icalproperty_get_recurrenceid (prop);
+
+  return icaltime_is_valid_time (ical_time) && !icaltime_is_null_time (ical_time) ? 
+    g_strdup (icaltime_as_ical_string (ical_time)) : NULL;
+}
+
 static char *
 get_ical_summary (icalcomponent *ical)
 {
@@ -529,18 +545,6 @@ get_ical_description (icalcomponent *ical)
   return g_strdup (icalproperty_get_description (prop));
 }
 
-static char *
-get_ical_url (icalcomponent *ical)
-{
-  icalproperty *prop;
-
-  prop = icalcomponent_get_first_property (ical, ICAL_URL_PROPERTY);
-  if (!prop)
-    return NULL;
-
-  return g_strdup (icalproperty_get_url (prop));
-}
-
 static inline GTime
 get_ical_start_time (icalcomponent *ical,
                      icaltimezone  *default_zone)
@@ -635,6 +639,18 @@ get_ical_completed_time (icalcomponent *ical,
                                  default_zone);
 }
 
+static int
+get_ical_priority (icalcomponent *ical)
+{
+  icalproperty *prop;
+
+  prop = icalcomponent_get_first_property (ical, ICAL_PRIORITY_PROPERTY);
+  if (!prop)
+    return -1;
+
+  return icalproperty_get_priority (prop);
+}
+
 static char *
 get_source_color (ECal *esource)
 {
@@ -751,6 +767,7 @@ calendar_appointment_init (CalendarAppointment  *appointment,
                            icaltimezone         *default_zone)
 {
   appointment->uid          = get_ical_uid (ical);
+  appointment->rid          = get_ical_rid (ical);
   appointment->summary      = get_ical_summary (ical);
   appointment->description  = get_ical_description (ical);
   appointment->color_string = get_source_color (source->source);
@@ -832,11 +849,11 @@ calendar_task_equal (CalendarTask *a,
     null_safe_strcmp (a->summary,      b->summary)      == 0 &&
     null_safe_strcmp (a->description,  b->description)  == 0 &&
     null_safe_strcmp (a->color_string, b->color_string) == 0 &&
-    null_safe_strcmp (a->url,          b->url)          == 0 &&
     a->start_time       == b->start_time                   &&
     a->due_time         == b->due_time                     &&
     a->percent_complete == b->percent_complete             &&
-    a->completed_time   == b->completed_time;
+    a->completed_time   == b->completed_time               &&
+    a->priority         == b->priority;
 }
 
 static void
@@ -850,11 +867,11 @@ calendar_task_copy (CalendarTask *task,
   task_copy->summary          = g_strdup (task->summary);
   task_copy->description      = g_strdup (task->description);
   task_copy->color_string     = g_strdup (task->color_string);
-  task_copy->url              = g_strdup (task->url);
   task_copy->start_time       = task->start_time;
   task_copy->due_time         = task->due_time;
   task_copy->percent_complete = task->percent_complete;
   task_copy->completed_time   = task->completed_time;
+  task_copy->priority         = task->priority;
 }
 
 static void
@@ -872,9 +889,6 @@ calendar_task_finalize (CalendarTask *task)
   g_free (task->color_string);
   task->color_string = NULL;
 
-  g_free (task->url);
-  task->url = NULL;
-
   task->percent_complete = 0;
 }
 
@@ -888,11 +902,11 @@ calendar_task_init (CalendarTask         *task,
   task->summary          = get_ical_summary (ical);
   task->description      = get_ical_description (ical);
   task->color_string     = get_source_color (source->source);
-  task->url              = get_ical_url (ical);
   task->start_time       = get_ical_start_time (ical, default_zone);
   task->due_time         = get_ical_due_time (ical, default_zone);
   task->percent_complete = get_ical_percent_complete (ical);
   task->completed_time   = get_ical_completed_time (ical, default_zone);
+  task->priority         = get_ical_priority (ical);
 }
 
 void
@@ -940,7 +954,8 @@ calendar_event_new (icalcomponent        *ical,
                           default_zone);
       break;
     default:
-      g_warning ("Unknown calendar component type\n");
+      g_warning ("Unknown calendar component type: %d\n",
+                 icalcomponent_isa (ical));
       g_free (event);
       return NULL;
     }
@@ -978,16 +993,16 @@ calendar_event_copy (CalendarEvent *event)
   return retval;
 }
 
-static const char *
+static char *
 calendar_event_get_uid (CalendarEvent *event)
 {
   switch (event->type)
     {
     case CALENDAR_EVENT_APPOINTMENT:
-      return CALENDAR_APPOINTMENT (event)->uid;
+      return g_strdup_printf ("%s%s", CALENDAR_APPOINTMENT (event)->uid, CALENDAR_APPOINTMENT (event)->rid ? CALENDAR_APPOINTMENT (event)->rid : ""); 
       break;
     case CALENDAR_EVENT_TASK:
-      return CALENDAR_TASK (event)->uid;
+      return g_strdup (CALENDAR_TASK (event)->uid);
       break;
     default:
       g_assert_not_reached ();
@@ -1195,14 +1210,14 @@ calendar_client_handle_query_completed (CalendarClientSource *source,
 
 static void
 calendar_client_handle_query_result (CalendarClientSource *source,
-				     GSList               *objects,
+				     GList                *objects,
 				     ECalView             *view)
 {
   CalendarClientQuery *query;
   CalendarClient      *client;
   gboolean             emit_signal;
   gboolean             events_changed;
-  GSList              *l;
+  GList               *l;
   GTime                month_begin;
   GTime                month_end;
 
@@ -1211,7 +1226,7 @@ calendar_client_handle_query_result (CalendarClientSource *source,
   query = goddamn_this_is_crack (source, view, &emit_signal);
 
   dprintf ("Query %p result: %d objects:\n",
-	   query, g_slist_length (objects));
+	   query, g_list_length (objects));
 
   month_begin = make_time_for_day_begin (1,
 					 client->priv->month,
@@ -1227,8 +1242,12 @@ calendar_client_handle_query_result (CalendarClientSource *source,
       CalendarEvent *event;
       CalendarEvent *old_event;
       icalcomponent *ical = l->data;
-
+      char          *uid;
+      
       event = calendar_event_new (ical, source, client->priv->zone);
+      if (!event)
+	      continue;
+
       calendar_event_generate_ocurrences (event,
 					  ical,
 					  source->source,
@@ -1236,21 +1255,24 @@ calendar_client_handle_query_result (CalendarClientSource *source,
 					  month_end,
                                           client->priv->zone);
 
-      old_event = g_hash_table_lookup (query->events,
-				       icalcomponent_get_uid (ical));
+      uid = calendar_event_get_uid (event);
+      
+      old_event = g_hash_table_lookup (query->events, uid);
 
       if (!calendar_event_equal (event, old_event))
 	{
-	  dprintf ("Event %s: ", old_event ? "modified" : "added");
+ 	  dprintf ("Event %s: ", old_event ? "modified" : "added");
 
 	  calendar_event_debug_dump (event);
 
-	  g_hash_table_replace (query->events,
-				(char *) calendar_event_get_uid (event),
-				event);
+	  g_hash_table_replace (query->events, uid, event);
 
 	  events_changed = TRUE;
 	}
+      else
+	{
+	  g_free (uid);
+	}		
     }
 
   if (emit_signal && events_changed)
@@ -1259,25 +1281,57 @@ calendar_client_handle_query_result (CalendarClientSource *source,
     }
 }
 
+static gboolean
+check_object_remove (gpointer key,
+                     gpointer value,
+                     gpointer data)
+{
+  char             *uid = data;
+  ssize_t           len;
+
+  len = strlen (uid);
+  
+  if (len <= strlen (key) && strncmp (uid, key, len) == 0)
+    {
+      dprintf ("Event removed: ");
+
+      calendar_event_debug_dump (value);
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 static void
 calendar_client_handle_objects_removed (CalendarClientSource *source,
-					GSList               *uids,
+					GList                *ids,
 					ECalView             *view)
 {
   CalendarClientQuery *query;
   gboolean             emit_signal;
   gboolean             events_changed;
-  GSList              *l;
+  GList               *l;
 
   query = goddamn_this_is_crack (source, view, &emit_signal);
 
   events_changed = FALSE;
-  for (l = uids; l; l = l->next)
+  for (l = ids; l; l = l->next)
     {
-      CalendarEvent *event;
-      const char    *uid = l->data;
+      CalendarEvent   *event;
+      ECalComponentId *id = l->data;
+      char            *uid = g_strdup_printf ("%s%s", id->uid, id->rid ? id->rid : "");
 
-      if ((event = g_hash_table_lookup (query->events, uid)))
+      if (!id->rid || !(*id->rid))
+	{
+	  int size = g_hash_table_size (query->events);
+
+	  g_hash_table_foreach_remove (query->events, check_object_remove, id->uid);
+
+		if (size != g_hash_table_size (query->events))
+			events_changed = TRUE;		
+	}
+      else if ((event = g_hash_table_lookup (query->events, uid)))
 	{
 	  dprintf ("Event removed: ");
 
@@ -1287,6 +1341,7 @@ calendar_client_handle_objects_removed (CalendarClientSource *source,
 
 	  events_changed = TRUE;
 	}
+      g_free (uid);
     }
 
   if (emit_signal && events_changed)
@@ -1362,7 +1417,7 @@ calendar_client_start_query (CalendarClient       *client,
   source->in_progress_query.events =
     g_hash_table_new_full (g_str_hash,
 			   g_str_equal,
-			   NULL,
+			   g_free,
 			   (GDestroyNotify) calendar_event_free);
 
   g_signal_connect_swapped (view, "objects-added",
@@ -1401,10 +1456,8 @@ calendar_client_update_appointments (CalendarClient *client)
 					  client->priv->month + 1,
 					  client->priv->year);
 
-  /* FIXME: occur-in-time-range should take recurrences into account */
-  query = g_strdup_printf ("(or (occur-in-time-range? (make-time \"%s\") "
-			                             "(make-time \"%s\")) "
-			       "(has-recurrences?))",
+  query = g_strdup_printf ("occur-in-time-range? (make-time \"%s\") "
+			                        "(make-time \"%s\")",
 			   month_begin, month_end);
 
   for (l = client->priv->appointment_sources; l; l = l->next)
@@ -1865,11 +1918,11 @@ calendar_client_foreach_appointment_day (CalendarClient  *client,
           if (day_time >= month_begin)
             marked_days [day_from_time_t (day_time)] = TRUE;
       
-          if (appointment->end_time)
+          if (appointment->end_time && !appointment->is_all_day)
             {
               int day_offset;
               int duration = appointment->end_time - appointment->start_time;
-              for (day_offset = 1; day_offset < duration / 86400; day_offset++)
+              for (day_offset = 1; day_offset <= duration / 86400; day_offset++)
                 {
                   GTime day_time = appointment->start_time + day_offset * 86400;
 
@@ -1979,26 +2032,3 @@ calendar_client_set_task_completed (CalendarClient *client,
 
   e_cal_modify_object (esource, ical, CALOBJ_MOD_ALL, NULL);
 }
-
-gboolean
-calendar_client_launch_editor (CalendarClient     *client,
-			       CalendarEventType   event_type,
-			       GdkScreen          *screen,
-			       GError            **error)
-{
-  char     *command_line;
-  gboolean  retval;
-
-  g_return_val_if_fail (CALENDAR_IS_CLIENT (client), FALSE);
-  g_return_val_if_fail (event_type == CALENDAR_EVENT_APPOINTMENT ||
-			event_type == CALENDAR_EVENT_TASK, FALSE);
-
-  command_line = g_strdup_printf ("evolution -c %s",
-				  event_type == CALENDAR_EVENT_APPOINTMENT ? "calendar" : "tasks");
-
-  retval = gdk_spawn_command_line_on_screen (screen, command_line, error);
-
-  g_free (command_line);
-
-  return retval;
-}
