@@ -21,7 +21,6 @@
 
 #include <gtk/gtkdialog.h>
 #include <libgnomeui/gnome-help.h>
-#include <libgnome/gnome-desktop-item.h>
 #include <glade/glade-xml.h>
 #include <libwnck/libwnck.h>
 #include <gconf/gconf-client.h>
@@ -203,8 +202,6 @@ applet_change_background (PanelApplet               *applet,
 						 GTK_RELIEF_NONE);
 		break;
 	}
-	wncklet_change_background (GTK_WIDGET (tasklist->applet), type,
-				   color, pixmap);
 }
 
 static void
@@ -732,6 +729,9 @@ window_list_applet_fill (PanelApplet *applet)
 			  "change_background",
 			  G_CALLBACK (applet_change_background),
 			  tasklist);
+
+	panel_applet_set_background_widget (PANEL_APPLET (tasklist->applet),
+					    GTK_WIDGET (tasklist->applet));
 	
 	panel_applet_setup_menu_from_file (PANEL_APPLET (tasklist->applet),
 					   NULL,
@@ -789,7 +789,7 @@ display_about_dialog (BonoboUIComponent *uic,
 			       authors,
 			       documenters,
 			       translator_credits,
-			       "panel-window-list",
+			       "gnome-panel-window-list",
 			       "tasklist",
 			       "Tasklist");
 }
@@ -1032,7 +1032,7 @@ display_properties_dialog (BonoboUIComponent *uic,
 	}
 
 	gtk_window_set_icon_name (GTK_WINDOW (tasklist->properties_dialog),
-				  "panel-window-list"); 
+				  "gnome-panel-window-list"); 
 
 	gtk_window_set_resizable (GTK_WINDOW (tasklist->properties_dialog), FALSE);
 	gtk_window_set_screen (GTK_WINDOW (tasklist->properties_dialog),
