@@ -31,6 +31,8 @@
 
 G_BEGIN_DECLS
 
+#define PANEL_MAIN_MENU_ICON "start-here"
+
 #define PANEL_TYPE_MENU_BUTTON         (panel_menu_button_get_type ())
 #define PANEL_MENU_BUTTON(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), PANEL_TYPE_MENU_BUTTON, PanelMenuButton))
 #define PANEL_MENU_BUTTON_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), PANEL_TYPE_MENU_BUTTON, PanelMenuButtonClass))
@@ -54,8 +56,9 @@ struct _PanelMenuButtonClass {
 
 GType      panel_menu_button_get_type            (void) G_GNUC_CONST;
 
-void       panel_menu_button_create              (PanelToplevel    *toplevel,
+gboolean   panel_menu_button_create              (PanelToplevel    *toplevel,
 						  int               position,
+						  const char       *filename,
 						  const char       *menu_path,
 						  gboolean          use_menu_path,
 						  const char       *tooltip);
@@ -78,9 +81,6 @@ void       panel_menu_button_load_from_gconf     (PanelWidget      *panel,
 						  gboolean          exactpos,
 						  const char       *id);
 
-void       panel_menu_button_save_to_gconf       (PanelMenuButton  *button,
-						  const char       *id);
-
 void       panel_menu_button_invoke_menu         (PanelMenuButton  *button,
 						  const char       *callback_name);
 
